--[[=============================================================================
    Functions for managing the status of the drivers bindings and connection state
 
    Copyright 2016 Control4 Corporation. All Rights Reserved.
===============================================================================]]
require "common_modified.c4_network_connection"
require "common_modified.c4_serial_connection"


-- This macro is utilized to identify the version string of the driver template version used.
if (TEMPLATE_VERSION ~= nil) then
	TEMPLATE_VERSION.connections = "2016.01.08"
end

-- constants
COM_USE_ACK = false
COM_COMMAND_DELAY_MILLISECONDS = 250
COM_COMMAND_RESPONSE_TIMEOUT_SECONDS = 4

NETWORK_PORT = 23
IR_BINDING_ID = 2
SERIAL_BINDING_ID = 1
NETWORK_BINDING_ID = 6001

--[[=============================================================================
    OnSerialConnectionChanged(idBinding, class, bIsBound)
  
    Description:
    Function called when a serial binding changes state(bound or unbound).
  
    Parameters:
    idBinding(int) - ID of the binding whose state has changed (SERIAL_BINDING_ID).
    class(string)  - Class of binding that has changed.
                     A single binding can have multiple classes(i.e. COMPONENT,
                     STEREO, RS_232, etc).
                     This indicates which has been bound or unbound.
    bIsBound(bool) - Whether the binding has been bound or unbound.
  
    Returns:
    None
===============================================================================]]

function OnSerialConnectionChanged(idBinding, class, bIsBound)
    if bIsBound == true then
	   C4:UpdateProperty ('Connected To Network', "False, Connected via Serial")
	   command = "telnetEcho 1"
	   LogTrace("command = " .. command)
	   PackAndQueueCommand("telnetEcho", command, 2, "SECONDS")
	   
	   gInitTimer:StartTimer()
    elseif bIsBound == false then
	   C4:UpdateProperty ('Connected To Network', "False")
	   C4:UpdateProperty ('Hardware Version', "")
	   C4:UpdateProperty ('Software Version', "")
	   C4:UpdateProperty ('Current Sensor A', "")
	   C4:UpdateProperty ('Hardware Version', "")
	   C4:UpdateProperty ('Software Version', "")
	   C4:UpdateProperty ('Mac Address', "")
	   C4:UpdateProperty ('Model', "")
	   C4:UpdateProperty ('Number of Outlets', "")
	   C4:UpdateProperty ('Serial Number', "")
	   C4:UpdateProperty ('Current Sensor B', "")
	   C4:UpdateProperty ('Humidity', "")
	   C4:UpdateProperty ('Temperature', "")
	   gTemperature = ""
	   gHumidity = ""
	   gCurrentA = ""
	   gCurrentB = ""
	   gModelHideUpdateProperties = ""
    end
end

--[[=============================================================================
    OnIRConnectionChanged(idBinding, class, bIsBound)
  
    Description:
    Function called when an IR binding changes state(bound or unbound).
  
    Parameters:
    idBinding(int) - ID of the binding whose state has changed (SERIAL_BINDING_ID).
    class(string)  - Class of binding that has changed.
                     A single binding can have multiple classes(i.e. COMPONENT,
                     STEREO, RS_232, etc).
                     This indicates which has been bound or unbound.
    bIsBound(bool) - Whether the binding has been bound or unbound.
  
    Returns:
    None
===============================================================================]]
function OnIRConnectionChanged(idBinding, class, bIsBound)
	
end

--[[=============================================================================
    OnNetworkConnectionChanged(idBinding, bIsBound)
  
    Description:
    Function called when a network binding changes state(bound or unbound).
  
    Parameters:
    idBinding(int) - ID of the binding whose state has changed.
    bIsBound(bool) - Whether the binding has been bound or unbound.
  
    Returns:
    None
===============================================================================]]
function OnNetworkConnectionChanged(idBinding, bIsBound)
    if bIsBound == false then
	   C4:UpdateProperty ('Hardware Version', "")
	   C4:UpdateProperty ('Software Version', "")
	   C4:UpdateProperty ('Current Sensor A', "")
	   C4:UpdateProperty ('Hardware Version', "")
	   C4:UpdateProperty ('Software Version', "")
	   C4:UpdateProperty ('Mac Address', "")
	   C4:UpdateProperty ('Model', "")
	   C4:UpdateProperty ('Number of Outlets', "")
	   C4:UpdateProperty ('Serial Number', "")
	   C4:UpdateProperty ('Current Sensor B', "")
	   C4:UpdateProperty ('Humidity', "")
	   C4:UpdateProperty ('Temperature', "")
	   gTemperature = ""
	   gHumidity = ""
	   gCurrentA = ""
	   gCurrentB = ""
	   gModelHideUpdateProperties = ""
    else 
	   gInitTimer:StartTimer()
    end
end

--[[=============================================================================
    OnNetworkStatusChanged(idBinding, nPort, sStatus)
  
    Description:
    Called when the network connection status changes. Sets the updated status of the specified binding
  
    Parameters:
    idBinding(int)  - ID of the binding whose status has changed
    nPort(int)      - The communication port of the specified bindings connection
    sStatus(string) - "ONLINE" if the connection status is to be set to Online,
                      any other value will set the status to Offline
  
    Returns:
    None
===============================================================================]]
function OnNetworkStatusChanged(idBinding, nPort, sStatus)
    if (sStatus == "OFFLINE") then
	   C4:UpdateProperty ('Current Sensor A', "")
	   C4:UpdateProperty ('Current Sensor B', "")
	   C4:UpdateProperty ('Temperature', "")
	   C4:UpdateProperty ('Humidity', "")
	   
	   gCurrentA = ""
	   gCurrentB = ""
	   gTemperature = ""
	   gHumidity = ""

	   gCon:StopPolling()
	   gCon._Priority2CommandQueue:clear()
    else
	   gCon:StartPolling(tonumber(gPollingTimerInterval), "SECONDS")
    end
end

--[[=============================================================================
    OnURLConnectionChanged(url)
  
    Description:
    Function called when the c4_url_connection is created.
  
    Parameters:
    url - url used by the url connection.
  
    Returns:
    None
===============================================================================]]
function OnURLConnectionChanged(url)
	
end

--[[=============================================================================
    DoEvents()
  
    Description:
    
  
    Parameters:
    None
  
    Returns:
    None
===============================================================================]]
function DoEvents()
end

--[[=============================================================================
    SendKeepAlivePollingCommand()
  
    Description:
    Sends a driver specific polling command to the connected system
  
    Parameters:
    None
  
    Returns:
    None
===============================================================================]]
function SendKeepAlivePollingCommand()
    PackAndQueueCommand("Display Network Status", "nwshow")
end

function OnPollingTimerExpired()
    PollProperties()
end

function PollProperties(bInit)
    LogInfo ("PollProperties()")
    
    gCon._Priority2CommandQueue:clear()
    
    PackAndQueuePriority2Command("Display_Power_Outlet_Status", CMDS.Display_Power_Outlet_Status)
    PackAndQueuePriority2Command("Read_Temperature_Sensor", CMDS.Read_Temperature_Sensor)
    PackAndQueuePriority2Command("Read_Current_A", CMDS.Read_Current_A)
    PackAndQueuePriority2Command("Read_Current_B", CMDS.Read_Current_B)
    PackAndQueuePriority2Command("Read_Humidity_Sensor", CMDS.Read_Humidity_Sensor)
    PackAndQueuePriority2Command("Show_Serial_Number", CMDS.Show_Serial_Number)
    
    if (bInit) then
	   PackAndQueuePriority2Command("Show_Version", CMDS.Show_Version)
	   PackAndQueuePriority2Command("Show_Mac", CMDS.Show_Mac)
    end
    
end

function OnPollingRestartTimerExpired()
    gPollingRestartTimer:KillTimer()
    PollProperties(true)
    gCon:StartPolling(tonumber(gPollingTimerInterval), "SECONDS")
end

function OnInitTimerExpired()
    gInitTimer:KillTimer()
    initialize()
end

function OnPowerOutletStatusDebounceTimerExpired()
    gPowerOutletStatusDebounceTimer:KillTimer()
    PackAndQueueCommand("Display_Power_Outlet_Status", CMDS.Display_Power_Outlet_Status)
end

function OnLoginTimerExpired()
    gLoginTimer:KillTimer()
    command = (Properties["Password"])
    PackAndQueuePriority1Command("UserPassword", command, 3, "SECONDS")
end

