--[[=============================================================================
    Get, Handle and Dispatch message functions

    Copyright 2015 Control4 Corporation. All Rights Reserved.
===============================================================================]]

-- This macro is utilized to identify the version string of the driver template version used.
if (TEMPLATE_VERSION ~= nil) then
	TEMPLATE_VERSION.device_messages = "2015.03.02"
end

--[[=============================================================================
    GetMessage()
  
    Description:
    Used to retrieve a message from the communication buffer. Each driver is
    responsible for parsing that communication from the buffer.
  
    Parameters:
    None
  
    Returns:
    A single message from the communication buffer
===============================================================================]]
function GetMessage()
    local message = ""
    --hexdump (gReceiveBuffer)
    
    if ((gReceiveBuffer ~= nil) and (gReceiveBuffer ~= "")) then
	   --special handling for login credentials--
	   
	   --is "enter password:" at end of gReceiveBuffer which would indicate that it is most likely the prompt
	   if (gReceiveBuffer:find("enter user name:", gReceiveBuffer:len() - 16) ~= nil) then DispatchMessage("username", "") end
	   
	   --is "enter password:" at end of gReceiveBuffer which would indicate that it is most likely the prompt
	   if (gReceiveBuffer:find("enter password:", gReceiveBuffer:len() - 15) ~= nil) then DispatchMessage("password", "") end
	   
	   --if login succeded we need to kill the PollingRestartTimer and restart the Polling Timer
	   if (gReceiveBuffer:find("login success") ~= nil) then 
		  gLoginAttempts = 0
		  OnPollingRestartTimerExpired() 
	   end
	   
    
    end
    
    local pattern = "(.+)>()"

    if (gReceiveBuffer:len() > 0) then
	   message, pos = string.match(gReceiveBuffer, pattern)
	   if (message == nil) then
		  --LOG:Info("Do not have a complete message")
		  return ""
	   end
	   gReceiveBuffer = gReceiveBuffer:sub(pos) 
    end

    return message
end

--[[=============================================================================
    HandleMessage(message)]

    Description
    This is where we parse the messages returned from the GetMessage()
    function into a command and data. The call to 'DispatchMessage' will use the
    'name' variable as a key to determine which handler routine, function, should
    be called in the DEV_MSG table. The 'value' variable will then be passed as
    a string parameter to that routine.

    Parameters
    message - Message string containing the function and value to be sent to DispatchMessage

    Returns
    Nothing
===============================================================================]]
function HandleMessage(message)

  if (type(message) == "table") then
    LogTrace("HandleMessage. Message is ==>")
    LogTrace(message)
    LogTrace("<==")
  else
    LogTrace("HandleMessage. Message is ==>%s<==", message)
  end

  -- TODO: Parse messages and DispatchMessage
  local name = ""
  local value = ""
  local bFirstLine = true
  local pattern = ""
  local pos = 1
  local line = ""
  local i = 0
  
  message = message:gsub(" ", "")
  for line in message:gmatch("(%C+)\r\n") do
    
    if (bFirstLine) then
--print("First Line: " .. line)
      --get header/message type
      if (line:find("temp") ~= nil) then
        name = "temp"
      elseif (line:find("ver") ~= nil) then
        name = "ver"
      elseif (line:find("cura") ~= nil) then
        name = "cura"
      elseif (line:find("curb") ~= nil) then
        name = "curb"
      elseif (line:find("ssn") ~= nil) then
        name = "ssn"
      elseif (line:find("mac") ~= nil) then
        name = "mac"
      elseif (line:find("rh") ~= nil) then
        name = "rh"
      elseif (line:find("reboot") ~= nil) then
        name = "reboot"
        return
      elseif (line:find("pshow") ~= nil) then
        name = "pshow"
      elseif (line:find("prb") ~= nil) then
        name = "prb"
      elseif (line:find("ps") ~= nil) then
        name = "ps"
      elseif (line:find("pset") ~= nil) then
        name = "pset"
      elseif (line:find("Port") ~= nil) then
        name = "port"
        gPortStatus = {}
      elseif (line:find("nwshow") ~= nil) then
        name = "nwshow"
      else
        LogTrace("Message not handled, aborting HandleMessage()")
        return
      end
      bFirstLine = false
    else
      --get status info
      i = i + 1
--print("Next Line: i = " .. i .. ", line = " .. line)
      if (name == "temp") then
        pattern = "Temperature:(.+)"
        value = line:match(pattern)
      elseif (name == "ver") then
        value = value .. line
      elseif (name == "cura") then
        pattern = "CurrentA:(.+)"
        value = line:match(pattern)
      elseif (name == "rh") then
        pattern = "Relative Humidity(RH):(.+)"  	--Relative Humidity(RH): 27.74% JAKE Need to parse this
	   value = value .. line
      elseif (name == "curb") then
        pattern = "CurrentB:(.+)"
        value = line:match(pattern)
      elseif (name == "ssn") then
        pattern = "SerialNO.:(.+)"
        value = line:match(pattern)
      elseif (name == "mac") then
        pattern = "(.+)"
        value = line:match(pattern)
      elseif (name == "reboot") then

      elseif (name == "pshow") then
        --only header info, not need to process
        return
      elseif (name == "port") then
--hexdump(line)
        pattern = "(.+)|(.+)|(.+)|(.+)|(.+)|(.+)" 
        local port, name, status, reserved_by, timer, auto_ping = line:match(pattern)
        if (port ~= nil) then
	   local t = {
		  ["port"] = port, 
		  ["name"] = name, 
		  ["status"] = status, 
		  ["reserved_by"] = reserved_by, 
		  ["timer"] = timer, 
		  ["auto_ping"] = auto_ping,
	   }
	   table.insert(gPortStatus, t)
	   end

        --value = line:match(pattern)
        --print(i,port, name, status, reserved_by, timer, auto_ping)
      end
      --print("i = " .. i .. ": " .. value)
    end
  end

  
  if (name == "port") then value = gPortStatus end
  
  name = name or message
  --value = value or ""
  --LogTrace("name = " .. name)
  --LogTrace("value = " .. value)

  DispatchMessage(name, value)

end




--[[=============================================================================
    DispatchMessage(MsgKey, MsgData)

    Description
    Parse routine that will call the routines to handle the information returned
    by the connected system.

    Parameters
    MsgKey(string)  - The function to be called from within DispatchMessage
    MsgData(string) - The parameters to be passed to the function found in MsgKey

    Returns
    Nothing
===============================================================================]]
function DispatchMessage(MsgKey, MsgData)
	if (DEV_MSG[MsgKey] ~= nil and (type(DEV_MSG[MsgKey]) == "function")) then
		--LogInfo("DEV_MSG.%s:  %s", MsgKey, MsgData)
		local status, err = pcall(DEV_MSG[MsgKey], MsgData)
		if (not status) then
			LogError("LUA_ERROR: %s", err)
		end
	else
		LogTrace("HandleMessage: Unhandled command = %s", MsgKey)
	end
end

function DEV_MSG.Level(level)
	gGenericProxy:dev_Level(level)
end

function DEV_MSG.port(value)
    local tLog = {}
    table.insert(tLog, "DEV_MSG.port" .. "\r")

    local bInit = false
    if (table.maxn(value) ~= gOutlets) then                --Check to determine if outlets are already set up.
	   bInit = true 
	   Outlets = {} -- create table to store the details of the outlets
	   Names = {} -- create table to store the details of the outlet names
	   gOutlets = table.maxn(value) -- As you now know the number of outlets store this in gOutlets
	   PersistData.gOutlets = gOutlets -- Also store this in the persist data table
	   PersistData.OutletBinding = {} --  create a persist data table for the outlet bindings we are about to create
    end 

    for j, k in pairs (value) do
	   local binding = tonumber(k.port) + 100
	   
	   if (bInit) then
		  table.insert(tLog, "Add Binding " .. binding .. " for " .. k.name .. "\r")
		  Names[binding] = k.name -- store the name for the outlet however you have this retrieved from the switch
		  PersistData["OutletBinding"][binding] = Names[binding] -- Store the name in the persist data table
		  C4:AddDynamicBinding(binding, "CONTROL", true, Names[binding] , "RELAY", false, false) -- this adds the binding to the project and names it to match the name of the outlet on the switch
	   end
	   
	   if (k.status == "ON") then -- here I am storing the initial status of the outlet as retreived by the switch
		  RelayToggleState [binding] = 1
	   else
		  RelayToggleState [binding] = 0
	   end
	   table.insert(tLog, "Notify Proxy: " .. k.name .. ", Binding" .. binding .. " is "  .. Notify [RelayToggleState [binding]] .. "\r")
	   C4:SendToProxy (binding, Notify [RelayToggleState [binding]], "", "NOTIFY") -- Notify Director of the initial state of the outlet
	   
	   -- Then we add variables for the data I can get for each outlet.  This may be different for your switch
	   --C4:AddVariable(Names[binding] .. ' current', "0", "NUMBER", true, false)
	   --C4:AddVariable(Names[binding] .. ' peak', "0", "NUMBER", true, false)
	   --C4:AddVariable(Names[binding] .. ' power', "0", "NUMBER", true, false)
	   --C4:AddVariable(Names[binding] .. ' alert', '0', "BOOL", true, false)
	   -- As my switch could have alerts for each outlet the line below dynamically adds an event for each outlet
	   --C4:AddEvent(100 + binding, Names[binding] .. ' alert', 'When NAME ' .. Names[binding] .. ' reports an alert')
    end
                
    local msg = table.concat(tLog) 
    LogTrace(msg)

end

function DEV_MSG.pshow(value)
    --nothing to do here
end

function DEV_MSG.nwshow(value)
    --do nothing here, used in keep-alive polling
end

function DEV_MSG.username(value) 
    LogTrace("DEV_MSG.username")
    
    gCon._Priority1CommandQueue:clear()
    gCon:StopPolling()
    gCon._Priority2CommandQueue:clear()
    gPollingRestartTimer:StartTimer()
    gReceiveBuffer = ""
    
    command = (Properties["Username"])
    PackAndQueuePriority1Command("UserName", command)
end

function DEV_MSG.password(value) 
    LogTrace("DEV_MSG.password")
    gLoginAttempts = gLoginAttempts + 1
    
    local command_predelay = 1
    if (gLoginAttempts > 10) then
	   command_predelay = 10
	   LogWarn("*\r\nWARNING: Unable to login, check Username and Password!!!\r\n*")
    elseif (gLoginAttempts > 6) then
	   if (gLoginAttempts == 7) then gCon:StopPolling() end
	   
	   command_predelay = 6
	   LogWarn("*\r\nWARNING: Unable to login, check Username and Password!!!\r\n*")
    elseif (gLoginAttempts > 4) then
	   command_predelay = 3
    else 
	   command_predelay = 1
    end
    
    gLoginTimer:StartTimer(command_predelay, "SECONDS")
    
end

function DEV_MSG.Login(value) 
    LogTrace("DEV_MSG.Login")
    C4:SendToNetwork (6001, 23, "login\r")
    gCon._CommandQueue:clear()
end

function DEV_MSG.pset(value)  -- The device reporting back state of port then trigger the pshow to pull state 
    LogTrace("DEV_MSG.pset") 
    PackAndQueueCommand("Power_Outlet_Status", CMDS.Display_Power_Outlet_Status)
end

function DEV_MSG.prb(value)
    LogTrace("DEV_MSG.prb")
    PackAndQueueCommand("Power_Outlet_Status", CMDS.Display_Power_Outlet_Status)
end

function DEV_MSG.ps(value)
    LogTrace("DEV_MSG.ps")  --port set message after telling the device to change an outlets state.  
    gPowerOutletStatusDebounceTimer:StartTimer()
end

function DEV_MSG.ver(value)
    LogTrace("DEV_MSG.ver")
    
    local pattern = "(...)(...)(...)(.....)()"
    local a, HW, b, SW, pos = string.match(value, pattern)

    C4:UpdateProperty ('Hardware Version', HW)
    C4:UpdateProperty ('Software Version', SW)
end

function DEV_MSG.ssn(value)
    LogTrace("DEV_MSG.ssn")
    C4:UpdateProperty ('Serial Number', value)
     
    if (value:find("P8") ~= nil) then
	   ModelHideUpdateProperties ("P8")
    elseif (value:find("P20") ~= nil) then
	   ModelHideUpdateProperties ("P20")
	   return
    elseif (value:find("P2") ~= nil) then
	   ModelHideUpdateProperties ("P2")
    end
end

function ModelHideUpdateProperties(value)
	LogTrace("ModelHideUpdateProperties ()")

	if (gModelHideUpdateProperties ~= value) then
	   gModelHideUpdateProperties = value
	    if (gModelHideUpdateProperties == "P2") then
		  C4:SetPropertyAttribs ("Current Sensor B min amps", 1) --hide property
		  C4:SetPropertyAttribs ("Current Sensor B max amps", 1)
		  C4:SetPropertyAttribs ("Current Sensor B", 1)	
		  C4:SetPropertyAttribs ("Temperature", 1)	
		  C4:SetPropertyAttribs ("Temperature Alert Threshold", 1)
		  C4:SetPropertyAttribs ("Relative Humidity", 1)
		  C4:UpdateProperty ('Model', "P2")
		  C4:UpdateProperty ('Number of Outlets', "2")
	    elseif (gModelHideUpdateProperties == "P8") then
		  C4:SetPropertyAttribs ("Current Sensor B min amps", 1) --hide property
		  C4:SetPropertyAttribs ("Current Sensor B max amps", 1)
		  C4:SetPropertyAttribs ("Current Sensor B", 1)
		  C4:SetPropertyAttribs ("Temperature", 0)	--show property
		  C4:SetPropertyAttribs ("Temperature Alert Threshold", 0)
		  C4:SetPropertyAttribs ("Relative Humidity", 0)
		  C4:UpdateProperty ('Model', "P8")
		  C4:UpdateProperty ('Number of Outlets', "8")
	    elseif (gModelHideUpdateProperties == "P20") then
		  C4:SetPropertyAttribs ("Current Sensor B min amps", 0) --show property
		  C4:SetPropertyAttribs ("Current Sensor B max amps", 0)
		  C4:SetPropertyAttribs ("Current Sensor B", 0)
		  C4:SetPropertyAttribs ("Temperature", 0)	
		  C4:SetPropertyAttribs ("Temperature Alert Threshold", 0)
		  C4:SetPropertyAttribs ("Relative Humidity", 0)
		  C4:UpdateProperty ('Model', "P20")
		  C4:UpdateProperty ('Number of Outlets', "20")
	   end
    end
end

function DEV_MSG.cura(value)  
    LogTrace("DEV_MSG.cura")
   
    if (value == nil) then return end
    
    local min = (tonumber2(Properties ["Current Sensor A min amps"]))
    local max =  (tonumber2(Properties["Current Sensor A max amps"])) 
    
    local valuenum = (string.gsub (value,"A",""))
   -- print (valuenum)
    valuenum = tonumber2(valuenum)
    --print (type(valuenum))
    
    
    if (gCurrentA ~= valuenum) then 
	   gCurrentA = valuenum
	   
	   C4:UpdateProperty ('Current Sensor A', valuenum)
	   C4:SetVariable("Current A", valuenum)
	   
	   if (min > valuenum) then
		  if (gCurra == false) then 
			 return 
		  end
		  gCurra = false
	   --print "minminminminminmin"
		  C4:FireEvent('Current Sensor A Min Exceeded')
	   elseif (max < valuenum) then
		  if (gCurra == true) then 
			 return 
		  end
		  gCurra = true
	   --print "maxmaxmaxmaxmaxmax"
		  C4:FireEvent('Current Sensor A Max Exceeded')
	   end
    end
end

function DEV_MSG.curb(value) 
    LogTrace("DEV_MSG.curb")

    if (value == nil) then return end
    
    local min = (tonumber2(Properties ["Current Sensor B min amps"]))
    local max =  (tonumber2(Properties["Current Sensor B max amps"])) 
    
    local valuenum = (string.gsub (value,"A",""))
   -- print (valuenum)
    valuenum = tonumber2(valuenum)
    --print (type(valuenum))
    
    
    if (gCurrentB ~= valuenum) then 
	   gCurrentB = valuenum
	   C4:UpdateProperty ('Current Sensor B', valuenum)
	   C4:SetVariable("Current B", valuenum)
	   
	   if (min > valuenum) then
		  if (gCurrb == false) then 
			 return 
		  end
		  gCurrb = false
		  --print "minminminminminmin"
		  C4:FireEvent('Current Sensor B Min Exceeded')
	   elseif (max < valuenum) then
		  if (gCurrb == true) then 
			 return 
		  end
		  gCurrb = true
		  --print "maxmaxmaxmaxmaxmax"
		  C4:FireEvent('Current Sensor B Max Exceeded')
	   end
    end
end

function DEV_MSG.temp (value)
    LogTrace "DEV_MSG.temp()" --, value = " .. value)
    
    if (value == nil) then return end
    
    local TempThreshold = (tonumber2(Properties["Temperature Alert Threshold"]))
    local valuenum = (string.gsub (value,"F",""))
   -- print (valuenum)
    valuenum = tonumber2(valuenum)
    --print (valuenum)
    
    
    if (gTemperature ~= valuenum) then 
	   gTemperature = valuenum
	   C4:FireEvent ('Temperature Changed')
	   C4:UpdateProperty ('Temperature', (valuenum.. " F"))
	   C4:SetVariable('Temperature', valuenum)
	   C4:SetVariable('SendTemp', valuenum)

	   if (TempThreshold > valuenum) then
	       if (gTemp == false) then 
			 return 
		  end
		  gTemp = false
		  --print "minminminminminmin"
		  C4:FireEvent ('Temperature Alert Under Threshold')
	   elseif (TempThreshold < valuenum) then
	       if gTemp == true then 
			 return 		  
		  end
		  gTemp = true
		  --print "maxmaxmaxmaxmaxmax"
		  C4:FireEvent ('Temperature Alert Over Threshold')
	   end
    end     
end

function DEV_MSG.mac(value)
    LogTrace("DEV_MSG.mac")
    C4:UpdateProperty ('Mac Address', value)
end

function DEV_MSG.rh(value)
    LogTrace("DEV_MSG.rh") --RelativeHumidity(RH):29.43%

    local humidity = (string.gsub (value,"%%",""))
    local x, temp, pos= string.match(humidity, "(.*):(.+)()")
    temp = (tonumber (temp))
    if (value == nil) then 
	   C4:UpdateProperty ('Relative Humidity', "Value is nill or sensor is not plugged in")
	   gHumidity = ""
	   return 
    end
    
    if (temp ~= nil) then
	   if gHumidity ~= temp then 
		  gHumidity = temp
		  C4:FireEvent ('Humidity Changed')
		  C4:UpdateProperty ('Relative Humidity', (temp .. "%"))
		  C4:SetVariable ("Humidity", temp)
	   end   
    end
end
